/**
 * Copyright (c) 2009-2011 Zauber S.A. <http://www.zaubersoftware.com/>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package ar.com.zauber.commons.mom.converter.impl;

import java.util.Date;
import java.util.GregorianCalendar;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;

import ar.com.zauber.commons.mom.converter.TypeConverter;

/**
 * Abstract implementation of {@link TypeConverter}s whose target type is
 * an {@link XMLGregorianCalendar}
 *
 * @author flbulgarelli
 */
public abstract class AbstractToXMLGregorianCalendarTypeConverter implements TypeConverter {

    private final DatatypeFactory datatypeFactory;
    {
        try {
            datatypeFactory = DatatypeFactory.newInstance();
        } catch (DatatypeConfigurationException e) {
            throw new AssertionError(e);
        }
    }

    /**
     * @see ar.com.zauber.commons.mom.converter.TypeConverter#canConvert(java.lang.Class, java.lang.Class)
     */
    public boolean canConvert(final Class<?> sourceType, final Class<?> destinationType) {
        return destinationType == XMLGregorianCalendar.class && canConvertSourceType(sourceType);
    }

    /**
     * if it's possible to convert the source object type to a XMLGregorianCalendar.
     * 
     * @param sourceType
     */
    protected abstract boolean canConvertSourceType(final Class<?> sourceType);

    /**
     * Converts a date into a {@link XMLGregorianCalendar}
     *
     * @param openingBalanceDate
     * @return an xml gregorian calendar that represents the given date
     */
    protected final XMLGregorianCalendar toGregorianCalendar(final Date date) {
        GregorianCalendar cal = new GregorianCalendar();
        cal.setTime(date);
        return datatypeFactory.newXMLGregorianCalendar(cal);
    }

}
