/**
 * Copyright (c) 2009-2011 Zauber S.A. <http://www.zaubersoftware.com/>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package ar.com.zauber.commons.mom.converter.impl;

import java.util.Date;

import net.sf.staccatocommons.restrictions.Constant;
import ar.com.zauber.commons.mom.converter.TypeConverter;

/**
 * {@link TypeConverter} that can convert {@link Date}s into {@link XMLGregorianCalendar}s
 * based on its time
 *
 * @author flbulgarelli
 */
public final class DateToXMLGregorianCalendarTypeConverter extends AbstractToXMLGregorianCalendarTypeConverter  {

    @Override
    @SuppressWarnings("unchecked")
    public <A> A convert(final Object value, final Class<A> destinationType) {
        return (A) toGregorianCalendar((Date) value);
    }

    @Override
    protected boolean canConvertSourceType(final Class<?> sourceType) {
        return Date.class.isAssignableFrom(sourceType);
    }

    /**
     * Answers a constant instance of {@link DateToXMLGregorianCalendarTypeConverter}
     * @return a constant {@link DateToXMLGregorianCalendarTypeConverter}
     */
    @Constant
    public static TypeConverter converter() {
        return new DateToXMLGregorianCalendarTypeConverter();
    }
}
