/**
 * Copyright (c) 2009-2011 Zauber S.A. <http://www.zaubersoftware.com/>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package ar.com.zauber.commons.mom.converter.impl.jodatime;

import ar.com.zauber.commons.mom.converter.TypeConverter;
import ar.com.zauber.commons.mom.converter.impl.AbstractToXMLGregorianCalendarTypeConverter;

import java.util.Date;

import net.sf.staccatocommons.restrictions.Constant;

import org.joda.time.format.DateTimeFormatter;
import org.joda.time.format.ISODateTimeFormat;

/**
 * {@link TypeConverter} that can convert a String into
 * {@link XMLGregorianCalendar}s, formatting it with a {@link DateTimeFormatter}
 *
 * @author flbulgarelli
 */
public final class StringToXMLGregorianCalendarTypeConverter extends
        AbstractToXMLGregorianCalendarTypeConverter {

    private final DateTimeFormatter formatter;

    /**
     * Creates a new {@link JodaStringToXMLGregorianCalendarTypeConverter}
     *
     * @param formatter
     */
    public StringToXMLGregorianCalendarTypeConverter(final DateTimeFormatter formatter) {
        this.formatter = formatter;
    }

    @SuppressWarnings("unchecked")
    @Override
    public <A> A convert(final Object value, final Class<A> destinationType) {
        return (A) toGregorianCalendar(parse(value));
    }

    /**
     * Parses a given date
     */
    private Date parse(final Object value) {
        return formatter.parseDateTime((String) value).toDate();
    }

    @Override
    protected boolean canConvertSourceType(final Class<?> sourceType) {
        return String.class == sourceType;
    }

    /**
     * Answers a {@link JodaStringToXMLGregorianCalendarTypeConverter} that uses
     * the given {@link DateTimeFormatter}
     *
     * @param format
     * @return a new {@link JodaStringToXMLGregorianCalendarTypeConverter}
     */
    public static TypeConverter from(final DateTimeFormatter formatter) {
        return new StringToXMLGregorianCalendarTypeConverter(formatter);
    }

    /**
     * Answers a constant {@link JodaStringToXMLGregorianCalendarTypeConverter}
     * that uses {@code ISODateTimeFormat.dateTimeParser()} as formatter
     *
     * @return a constant {@link TypeConverter}
     */
    @Constant
    public static TypeConverter isoDateTime() {
        return from(ISODateTimeFormat.dateTimeParser());
    }

}