/**
 * Copyright (c) 2009-2011 Zauber S.A. <http://www.zaubersoftware.com/>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package ar.com.zauber.commons.mom.internal;

import java.util.LinkedList;
import java.util.List;

import net.sf.staccatocommons.restrictions.check.NonNull;
import net.sf.staccatocommons.restrictions.check.NotEmpty;
import net.sf.staccatocommons.restrictions.processing.ForceRestrictions;
import ar.com.zauber.commons.mom.MapObjectMapper;
import ar.com.zauber.commons.mom.MapObjectMapperBuilder;
import ar.com.zauber.commons.mom.MapObjectMapperInterceptor;
import ar.com.zauber.commons.mom.converter.CompositeTypeConverter;
import ar.com.zauber.commons.mom.converter.IdentityTypeConverter;
import ar.com.zauber.commons.mom.converter.TypeConverter;
import ar.com.zauber.commons.mom.style.SetterStyle;
import ar.com.zauber.commons.mom.style.impl.BeanStyle;

/**
 *
 * DefaultMapObjectMapperBuilder
 * @author flbulgarelli
 */
public class DefaultMapObjectMapperBuilder implements MapObjectMapperBuilder {

    private final String packageName;
    private SetterStyle setterStyle = BeanStyle.STYLE;
    private List<TypeConverter> converters = new LinkedList<TypeConverter>();
    private MapObjectMapperInterceptor interceptor = null;

    public DefaultMapObjectMapperBuilder(@NotEmpty final String packageName) {
        this.packageName = packageName;
        withConverter(IdentityTypeConverter.converter());
    }

    /**
     * @see ar.com.zauber.commons.mom.MapObjectMapperBuilder#withSetterStyle(ar.com.zauber.commons.mom.style.SetterStyle)
     */
    @ForceRestrictions
    public MapObjectMapperBuilder withSetterStyle(@NonNull final SetterStyle style) {
        this.setterStyle = style;
        return this;
    }

    /**
     * @see ar.com.zauber.commons.mom.MapObjectMapperBuilder#withConverter(ar.com.zauber.commons.mom.converter.TypeConverter)
     */
    @ForceRestrictions
    public MapObjectMapperBuilder withConverter(@NonNull final TypeConverter converter) {
        this.converters.add(converter);
        return this;
    }

    /**
     * @see ar.com.zauber.commons.mom.MapObjectMapperBuilder#withInterceptor(ar.com.zauber.commons.mom.MapObjectMapperInterceptor)
     */
    @ForceRestrictions
    public MapObjectMapperBuilder withInterceptor(@NonNull final MapObjectMapperInterceptor inter) {
        this.interceptor = inter;
        return this;
    }

    /**
     * @see ar.com.zauber.commons.mom.MapObjectMapperBuilder#build()
     */
    public MapObjectMapper build() {
        if (interceptor == null) {
            return new DefaultMapObjectMapper(packageName, setterStyle, typeConverter());
        }
        return new InterceptingMapObjectMapper(packageName, setterStyle, typeConverter(), interceptor);
    }

    private CompositeTypeConverter typeConverter() {
        return new CompositeTypeConverter(converters);
    }
}
