/**
 * Copyright (c) 2009-2011 Zauber S.A. <http://www.zaubersoftware.com/>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package ar.com.zauber.commons.mom.internal;

import net.sf.staccatocommons.restrictions.check.NonNull;

/**
 * An object for determining if objects of certain classes can be mapped into
 * maps
 */
public final class MappingPackage {
    private final String packageName;

    /**
     * Creates the mapping package
     * @param packageName
     */
    public MappingPackage(@NonNull final String packageName) {
        this.packageName = packageName;
    }

    public boolean isMappeable(@NonNull final Class<?> clazz) {
        return !clazz.isPrimitive() && !clazz.isArray() && packageContains(clazz);
    }

    protected boolean packageContains(final Class<?> clazz) {
        return clazz.getName().startsWith(packageName) //
                || clazz.getSuperclass() != null && clazz.getSuperclass().getName().startsWith(packageName);
    }
}
