/**
 * Copyright (c) 2009-2011 Zauber S.A. <http://www.zaubersoftware.com/>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package ar.com.zauber.commons.mom.internal;

import java.beans.PropertyDescriptor;
import java.lang.reflect.Method;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.util.List;

import net.sf.staccatocommons.check.Ensure;
import net.sf.staccatocommons.lang.SoftException;
import net.sf.staccatocommons.restrictions.check.NonNull;
import net.sf.staccatocommons.restrictions.check.NotEmpty;
import net.sf.staccatocommons.restrictions.processing.ForceRestrictions;

import org.apache.commons.beanutils.PropertyUtils;

/**
 * StructureType
 * @author flbulgarelli
 */
public final class StructureType {

    /**Utility class**/
    private StructureType() {
    }

    private static Type getType(final Object bean, final String propertyName) {
        PropertyDescriptor propertyDescriptor;
        try {
            propertyDescriptor = PropertyUtils.getPropertyDescriptor(bean, propertyName);
        } catch (Exception e) {
            throw SoftException.soften(e);
        }
        Ensure.that(propertyDescriptor != null,
                "There is no property named %s in class %s",
                propertyName,
                bean.getClass());

        if(propertyDescriptor.getPropertyType() == Boolean.class) {
            return Boolean.class;
        }
        Method readMethod = propertyDescriptor.getReadMethod();
        Ensure.that(readMethod != null,
                "Property %s in class %s is not readable",
                propertyName,
                bean.getClass());
        return readMethod.getGenericReturnType();
    }

    @ForceRestrictions
    @SuppressWarnings("unchecked")
    public static Class getStructureType(@NonNull final Object destination, @NotEmpty final String propertyName) {
        Type targetType = getType(destination, propertyName);


        if (targetType instanceof Class) {
            return (Class) targetType;
        }

        if (targetType instanceof ParameterizedType) {
            ParameterizedType parameterizedType = (ParameterizedType) targetType;
            if (parameterizedType.getRawType() == List.class) {
                return (Class) parameterizedType.getActualTypeArguments()[0];
            }
        }
        return Ensure.that().fail(
                "Unsupported type %s for propertyName %s in object %s",
                targetType, (Object) propertyName, destination);
    }
}
