/**
 * Copyright (c) 2009-2011 Zauber S.A. <http://www.zaubersoftware.com/>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/*
 * Copyright (c) 2011 Zauber S.A. -- All rights reserved
 */

package ar.com.zauber.commons.mom.style.impl;

import java.lang.reflect.InvocationTargetException;
import java.util.List;

import net.sf.staccatocommons.restrictions.check.NonNull;
import net.sf.staccatocommons.restrictions.check.NotEmpty;
import net.sf.staccatocommons.restrictions.processing.ForceRestrictions;

import org.apache.commons.beanutils.PropertyUtils;

import ar.com.zauber.commons.mom.style.SetterStyle;

/**
 * The CXF Style of setting fields.
 * 
 * @author Gaston Ponti
 * @since Aug 26, 2011
 */
@SuppressWarnings("unchecked")
public class CXFStyle extends BeanStyle {

    /**
     * An instance of {@link CXFStyle}
     */
    public static final SetterStyle STYLE = new CXFStyle();

    /**
     * @throws NoSuchMethodException
     * @throws InvocationTargetException
     * @throws IllegalAccessException
     * @see
     *      ar.com.zauber.commons.mom.PropertyStyle#isSeteable(org.apache.commons
     *      .beanutils.BeanUtilsBean,
     *      org.apache.commons.beanutils.Map<String,Object>,
     *      org.apache.commons.beanutils.Object, java.lang.String)
     */
    @Override
    @ForceRestrictions
    public boolean isSeteable(@NonNull final Object destination,
            @NotEmpty final String propertnName, final Object propertyValue)
            throws Exception {
        return super.isSeteable(destination, propertnName, propertyValue)
                || isListWithGetter(destination, propertnName);
    }

    private boolean isListWithGetter(final Object destination, final String propertyName)
            throws IllegalAccessException, InvocationTargetException,
            NoSuchMethodException {
        return PropertyUtils.isReadable(destination, propertyName) && isPropertyTypeList(
                destination, propertyName);
    }

    private boolean isPropertyTypeList(final Object destination, final String propertyName)
            throws IllegalAccessException, InvocationTargetException,
            NoSuchMethodException {
        return List.class.isAssignableFrom(PropertyUtils.getPropertyType(
                destination, propertyName));
    }

    @Override
    @ForceRestrictions
    public void setValue(@NonNull final Object destination, @NotEmpty final String name,
            final Object property) throws Exception {
        if (isPropertyTypeList(destination, name)) {
            if (property != null) {
                getListProperty(destination, name).addAll((List) property);
            }
        } else {
            super.setValue(destination, name, property);
        }
    }

    public List getListProperty(final Object destination, final String name)
            throws IllegalAccessException, InvocationTargetException,
            NoSuchMethodException {
        return (List) PropertyUtils.getProperty(destination, name);
    }

}